install.packages(c("jsonlite","tidyr","dplyr","ggplot2","magrittr","hrbrthemes","stringr",
                   "patchwork","ggtext","forcats","svglite"))

library(jsonlite)
library(tidyr)
library(dplyr)
library(ggplot2)
library(magrittr)
library(hrbrthemes)
library(stringr)
library(patchwork)
library(ggtext)
library(forcats)

hrbrthemes::import_roboto_condensed() 
my_font <- "Roboto Condensed"
accent_color <- "#e59635"

#Read json files
datasets <- fromJSON(txt="datasets.json")

group_dict_url <- "https://raw.githubusercontent.com/4TUResearchData/djehuty/main/src/djehuty/backup/resources/groups.json"
group_dict <- fromJSON(group_dict_url) %>% select(id,group_name = name)

#Read views / downloads
views_downloads <- read.table("views_downloads.tsv",sep = "\t", header = TRUE)

##################################################################
##                      DATA PREPROCESSING                      ##
##################################################################

#Convert to dataframe and remove duplicates
datasets <- datasets %>% as.data.frame() %>% distinct() %>% arrange(title)

#Add name of institution
datasets <- datasets %>% left_join(group_dict,by=c("group_id"="id"))

#Separate date and time columns
datasets <- datasets %>% separate(published_date, c("published_date", "time"), sep = "T")
#Separate date into year / month / day
datasets <- datasets %>% separate(published_date, c("year", "month", "day"), sep = "-",remove = FALSE) %>% mutate(year=as.numeric(year), published_date=as.Date(published_date))

#Add views and downloads
datasets <- datasets %>% left_join(views_downloads, by="uuid")

#Inspect elements that don't have a year
datasets %>% filter(is.na(year)) %>% group_by(embargo_title,embargo_date) %>% summarise(n=n())
#It's only 8 elements out of 8,593... we will ignore these for the visualizations
datasets <- datasets %>% filter(!is.na(year))

#We visualize the datasets published per year
datasets %>% group_by(year) %>% summarise(n=n()) %>% ggplot(aes(x=year,y=n))+geom_bar(stat="identity")

#There is something odd in 2011... when investigating the year, turns out that some of the entries in the data should belong to a "unique" dataset
datasets %>% filter(year==2011) %>% View()
#For example, the IDRA dataset is spread into 282 entries...
datasets %>% filter(year==2011,grepl("IDRA weather radar measurements",title)) %>% nrow()

##################################################################
##                   PSEUDO-DUPLICATE REMOVAL                   ##
##################################################################

#We now try to tackle this situation by grouping such entries into a single one
#We inspect cases where >90% of the title matches multiple entries....
datasets %>% mutate(title_new = substr(title, 1, nchar(title)*0.5)) %>% group_by(title_new) %>% summarise(n=n()) %>% filter(n>1) %>% arrange(n) %>% View()
#and group them together:
datasets <- datasets %>% mutate(title_new = substr(title, 1, nchar(title)*0.5))

#We give special treatment to two datasets: IDRA weather radar measurements and Maisbich temperature data:
datasets <- datasets %>% mutate(title_new=ifelse(grepl("IDRA weather radar measurements",title),"IDRA weather radar measurements",title_new))
datasets <- datasets %>% mutate(title_new=ifelse(grepl("Maisbich",title),"Maisbich sensor data",title_new))


#We now want to define a title for each parent dataset
# We will arbitrarily give the name of the first element to the dataset
datasets <- datasets %>% group_by(title_new) %>% mutate(title_new_complete=title[1]) %>% ungroup()

# Some string manipulations to obtain a column with "clean titles"
datasets <- datasets %>%  mutate(title_clean = tolower(title_new_complete)) %>% #Everything in lower case
  mutate(title_clean = str_replace_all(title_clean, '[^a-zA-Z0-9]',' ')) %>% #Replace special characters with space
  mutate(title_clean = str_squish(title_clean)) #Remove leading and trailing whitespaces (and repeated spaces)

# Finally, generating a new dataset with the clean titles and without duplicates
datasets_clean <- datasets %>% group_by(title_new,year) %>% slice_head(n=1) %>% ungroup() %>% select(title_clean,year,group_name)

#We validate the procedure... the new plot makes much more sense
datasets_histogram <- datasets_clean %>%  group_by(year) %>% summarise(n=n()) %>% ungroup
datasets_histogram %>% ggplot(aes(x=year,y=n))+geom_bar(stat="identity")

#There are still duplicates and further treatment could be done... 
#As the competition focuses on visuals, we will not treat the remaining cases 

#We manually change the name to some group categories...
datasets_clean <- datasets_clean %>% mutate(group_name = case_when(grepl("Delft",group_name) ~ "Delft University of Technology",
                                                                   grepl("Eindhoven",group_name) ~ "Eindhoven University of Technology",
                                                                   grepl("Twente",group_name) ~ "University of Twente",
                                                                   TRUE ~ group_name)) 

#################################################################
##          1st VISUALIZATION - SHARE BY ORGANIZATION          ##
#################################################################

university_count <- datasets_clean %>% group_by(group_name) %>% summarise(n=n()) %>% ungroup() %>% arrange(desc(n))

#Everything with less than 100 datasets is sent to "Other institutions"
university_count <- university_count %>% mutate(group_name = ifelse(n<100,"Other institutions",group_name))

#Count per university
university_count <- university_count %>% group_by(group_name) %>% summarise(n=sum(n)) %>% ungroup() %>% arrange(desc(n))

#Changing the order according to the number of items
university_count$group_name <- fct_reorder(university_count$group_name, university_count$n)

#Adding a column with the cumulative percentage (to plot the labels)
university_count <- university_count %>% mutate(n_cum = cumsum(n)) %>% mutate(cum_p = n_cum/sum(n))

#Building the plot
plot_1 <- university_count %>%  ggplot(aes(fill=group_name, y=n, x="")) + 
  geom_bar(position="fill", stat="identity", alpha = 0.8)+
  coord_flip(clip = "off")+ #allow labels to extend beyond plot limits 
  guides(fill="none")+ #remove legend for fill, as it is in the title
  scale_fill_manual(values=c("#8a2118","#d92f7d","#306d1e","gray20","#00A6D6"))+
  scale_y_percent()+ #x axis as percentage
  labs(title = "The repository hosts over <span style='color:#e59635;font-size:15px;'>**8,500**</span> datasets from leading research institutions, including:<br>
       <span style='color:#00A6D6;font-size:14px;'>**TU Delft**</span>,
       <span style='color:#306d1e;font-size:14px;'>**Wageningen University & Research**</span>,
       <span style='color:#d92f7d;font-size:14px;'>**University of Twente**</span>,
       <span style='color:#8a2118;font-size:14px;'>**TU Eindhoven**</span>,
       <span style='color:#5c5c5c;font-size:14px;'>**and +10 others**</span>",
       subtitle = "Share of the number of databases per institution")+
  theme_void()+
  theme(plot.title = element_markdown(size = 12, family = my_font),
        plot.subtitle = element_text(size = 10, family = my_font,hjust = 1),
        axis.text.x = element_text(size = 8),
        plot.margin = unit(c(0.75,0.5,0,0), "cm"))

#################################################################
##                2nd VISUALIZATION - HISTOGRAM                ##
#################################################################

#Now on to the visuals!
#First we will do a custom version of a histogram (count of the number of different datasets published each year)


#Experimenting with polygons to create a paper sheet...
#First page
edges <- data.frame(
  id = 1,
  x = c(1.1, 1.5, 0.9, 0.5),
  y = c(1.4, 1.05, 0.6, 0.95))
#Page on top...
edges <- edges %>% bind_rows(data.frame(
  id = 2,
  x = c(1.1, 1.5, 0.9, 0.5),
  y = c(1.4, 1.05, 0.6, 0.95)+0.1))

#Plot both pages:
edges %>% ggplot(aes(x = x, y = y)) + geom_polygon(aes(group=id),fill="white",color="black")

#Looks good! Now we use this custom shape in our histogram

#Extrapolation of the number of datasets for 2023...
days_so_far = difftime(max(datasets$published_date),as.Date("2023-01-01")) %>% as.numeric()
datasets_so_far = datasets_histogram[datasets_histogram$year==2023,]$n 
expected_datasets = (365 * datasets_so_far/ days_so_far) %>% as.integer()
datasets_histogram["n"][datasets_histogram["year"] == 2023] <- expected_datasets


#Each page will represent 5 datasets
datasets_histogram <- datasets_histogram %>% mutate(n=n/5)
#We create one row per "page"
datasets_histogram <- datasets_histogram  %>% group_by(year,n) %>% reframe(page = as.integer(seq(1, n, 1)))
#And finally, one row per edge
datasets_histogram <- datasets_histogram %>% slice(rep(1:n(), each = 4)) %>%
  group_by(year,n,page) %>% mutate(edge = dplyr::row_number())
#We calculate the coordinates for the page edges
datasets_histogram <- datasets_histogram %>% mutate(y = case_when(edge==1 ~ page+1.1,
                                                                  edge==2 ~ page+0.05,
                                                                  edge==3 ~ page-1.1,
                                                                  edge==4 ~ page-0.05))

datasets_histogram <- datasets_histogram %>%  mutate(x = case_when(edge==1 ~ year+0.05,
                                                                   edge==2 ~ year+0.15,
                                                                   edge==3 ~ year-0.05,
                                                                   edge==4 ~ year-0.15))

#We use a different color to indicate the value we extrapolated for 2023...
datasets_histogram <- datasets_histogram %>% mutate(color = ifelse(year == 2023 & page > (datasets_so_far/5),"gray40", "#5386c5"))

#Generate the second plot
plot_2 <- datasets_histogram %>% ggplot(aes(x = x, y = y)) + 
  geom_polygon(aes(group=paste(year,sprintf("%03d", page)),color=color),fill="white", linewidth=0.3)+ #paper sheets
  coord_cartesian(clip="off")+ #allow labels to extend beyond plot limits
  scale_color_identity()+ #use colors within dataset
  scale_y_continuous(labels=function(x)x*5, limits=c(-1,200))+ #Each page represents 5 datasets
  scale_x_continuous(breaks = seq(2009,2023))+ #years within data
  annotate("text", y = 810/5, x = 2023.3, label = "Projected value", angle = 90,size=2.5)+
  annotate("text", y = 800/5, x = 2021, label = "COVID-19 effect?", size=3, color = accent_color)+
  labs(x="Year", 
       y= "Number of datasets",
       subtitle = "Each paper sheet represents 5 datasets",
       title = "Since <span style='color:#e59635;font-size:15px;'>**2009**</span>, the number of datasets on the repository has grown exponentially",
       caption = "*Similar datasets are counted as a single one, based on name similarities")+
  theme(plot.title = element_markdown(size = 12, family = my_font,face = "plain"),
        plot.subtitle = element_text(size = 10, family = my_font,hjust = 1),
        axis.text.x = element_text(size = 8),
        axis.title =  element_text(size = 9, family = my_font,hjust = 1),
        plot.margin = unit(c(0.75,0.5,0.5,0), "cm"))

  
#################################################################
##               3rd VISUALIZATION: VIEWS VS AGE               ##
#################################################################

#Create new variable representing the "age" of each dataset
datasets <- datasets %>% mutate(days_in_rep= as.Date(Sys.Date()) - published_date, days_in_rep = as.numeric(days_in_rep)) 

#Get the average number of downloads per day
avg_downloads_day <- sum(datasets$views,na.rm = TRUE)/sum(datasets$days_in_rep)

#Generate the plot
plot_3 <- datasets%>% ggplot(aes(x=days_in_rep,y=views))+
  geom_point(color="#5386c5")+
  scale_y_comma()+ #Thousands separator
  scale_x_comma()+ #Thousands separator
  geom_abline(slope=avg_downloads_day, intercept=0, color = accent_color, linewidth = 1.2)+
  annotate("text", y = 3500, x = 4500, label = "Average behaviour", size=3, color = accent_color, angle = 3)+
  annotate("text", y = 18000, x = 3000, label = "Data for the BPI Challenge", size=3, color = accent_color)+
  annotate("text", y = 16000, x = 930, label = "Fake News Detection Dataset", size=3, color = accent_color)+
  annotate("text", y = 16000, x = 4578, label = "Hospital Logs", size=3, color = accent_color)+
  annotate("segment", x = c(2450,3000,3550), xend = c(1770,2500,4100), y = c(18600,18600,18600), yend = c(21000,19700,19100),color = accent_color)+
  labs(x="Days online", 
       y= "Number of views",
       subtitle = "Each point represents a dataset",
       title = "On average, each dataset is viewed <span style='color:#e59635;font-size:15px;'>**0.3**</span> times per day.<br>
       There are however some entries that have received an outrageous amount of attention: ")+
  theme(plot.title = element_markdown(size = 12, family = my_font,face = "plain"),
        plot.subtitle = element_text(size = 10, family = my_font,hjust = 1),
        axis.text.x = element_text(size = 8),
        axis.title =  element_text(size = 9, family = my_font,hjust = 1),
        plot.margin = unit(c(0.75,0.5,0.5,0), "cm"))
#datasets%>% ggplot(aes(x=days_in_rep,y=downloads))+geom_point()


##################################################################
##          4th VISUALIZATION: POPULAR TERMS EVOLUTION          ##
##################################################################

#Split sentences into words
word_by_word <- datasets_clean %>% group_by(year) %>%  separate_longer_delim(title_clean,delim=" ") %>% rename(word=title_clean)

#Get count of words per year
word_count_per_year <- word_by_word %>% group_by(year,word) %>% summarise(n=n()) %>% ungroup()
word_count_per_year <- word_count_per_year %>% arrange(year,desc(n))

#We explore the data looking for interesting term-trends...
word_count_per_year %>% filter(year==2023) %>% View()
word_count_per_year %>% filter(word %in% c("covid","covid19"))

#Group "families" of terms into single terms
word_count_per_year <- word_count_per_year %>% mutate(word_new = case_when(word %in% c("covid","covid19") ~ "covid",
                                                                           grepl("water", word) ~ "water",
                                                                           grepl("econo", word) ~ "economy",
                                                                           word %in% c("dutch","netherlands") ~ "netherlands",
                                                                           TRUE ~ word)) 

#Generate plot
plot_4 <- word_count_per_year %>% group_by(word_new, year) %>% summarise(n=sum(n)) %>% ungroup() %>% #Recalculate number, considering term families
  filter(word_new %in% c("covid","water","netherlands")) %>% #only include these in plot
  ggplot(aes(x=year,y="")) + 
  geom_point(aes(size=n), color = "#5386c5")+
  scale_x_continuous(limits=c(2009,2023), breaks = seq(2009,2023))+
  facet_grid(word_new ~ .)+ #create a facet per term
  coord_cartesian(clip = "off")+
  scale_size_continuous(name="")+
  theme_minimal()+
  labs(x="Year", 
       y= "Term",
       subtitle = "Circle size depicts the number of datasets uploaded each year containing each term in the title",
       title = "The number of datasets through the years reflects the trendiness of research topics")+
  theme(plot.title = element_markdown(size = 12, family = my_font,face = "plain"),
        plot.subtitle = element_text(size = 10, family = my_font,hjust = 1),
        axis.text.x = element_text(size = 8),
        axis.title =  element_text(size = 9, family = my_font,hjust = 1),
        plot.margin = unit(c(0.75,0.5,0.5,0), "cm"),
        strip.text = element_text(size = 6))
  


# Combine plots: 
final_plot <- plot_1 / plot_2 / plot_3 / plot_4+ #arrange the 4 plots vertically
  plot_layout(widths = c(1.5, 1.5, 1.5, 1.5), heights = unit(c(1, 9, 9, 5), c('cm', 'null'))) & #define relative plot sizes
  theme(plot.background = element_rect(fill = "#e5eafc", color = "transparent"),
        panel.background = element_rect(fill ="#e5eafc", color = "transparent"))

#Add title and attributions to final plot
final_plot +
  plot_annotation(title = "<span style='color:#e59635;font-size:30px;'>4TU</span>.Research Data: A Visual Summary",
                  caption = "Data: 4TU Research Data | Visualization: Rodrigo Revilla Llaca", 
                  theme = theme(plot.title = element_markdown(color = "#292929", size = 22, face = "bold", family = my_font, hjust = 0.5),
                                plot.caption = element_text(color = "grey50", family = my_font),
                                plot.margin = unit(c(1,0.2,0.2,1), "cm")))

ggsave("Rplot.svg",width=7.4, height=13.5, units = "in")
#Export resolution must be 740 x 1350, preferably in .svg format to preserve quality 